"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const checkCommandInputs_1 = require("../../helpers/checkCommandInputs");
class MethodsCommand extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(MethodsCommand);
        if (checkCommandInputs_1.isEmptyCommand(flags, args)) {
            this._help();
            this.exit(1);
        }
        try {
            const { abi } = args;
            const { getMethods } = await Promise.resolve().then(() => __importStar(require('../../helpers/abi')));
            const methods = getMethods(abi);
            methods.forEach(({ signature, signatureHash }) => {
                this.log(`${signatureHash}\t${signature}`);
            });
        }
        catch (e) {
            this.error(e.message, { exit: 1 });
        }
    }
}
exports.default = MethodsCommand;
MethodsCommand.description = `Get the hash of each method in the given ABI.`;
MethodsCommand.args = [
    {
        name: 'abi',
        required: false,
        description: 'Contract ABI.',
    },
];
MethodsCommand.examples = ['eth abi:methods ../contracts/proxy.abi'];
MethodsCommand.aliases = ['me'];
