"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const checkCommandInputs_1 = require("../../helpers/checkCommandInputs");
const knownAbis_1 = require("../../helpers/knownAbis");
class ShowCommand extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(ShowCommand);
        if (checkCommandInputs_1.isEmptyCommand(flags, args)) {
            this._help();
            this.exit(1);
        }
        const { abi } = args;
        const abiObj = knownAbis_1.getAbiByName(abi);
        if (abiObj) {
            cli_ux_1.cli.styledJSON(abiObj);
        }
        else {
            this.error(`ABI for ${abi} not found!`);
        }
    }
}
exports.default = ShowCommand;
ShowCommand.description = 'Displays a known ABI';
ShowCommand.flags = {
    help: command_1.flags.help({ char: 'h' }),
};
ShowCommand.args = [
    {
        name: 'abi',
        required: false,
        description: 'The contract name.',
    },
];
ShowCommand.examples = ['eth abi:show ERC20', 'eth abi:show ERC721'];
