"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const checkCommandInputs_1 = require("../../helpers/checkCommandInputs");
class AddressCommand extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(AddressCommand);
        if (checkCommandInputs_1.isEmptyCommand(flags, args)) {
            this._help();
            this.exit(1);
        }
        try {
            const { account, nonce } = args;
            const { getContractAddress } = await Promise.resolve().then(() => __importStar(require('../../helpers/getContractAddress')));
            const contractAddress = getContractAddress(account, nonce);
            this.log(contractAddress);
        }
        catch (e) {
            this.error(e.message, { exit: 1 });
        }
    }
}
exports.default = AddressCommand;
AddressCommand.description = `Get the address for a contract created from the given <account> with the given <nonce>.`;
AddressCommand.args = [
    {
        name: 'account',
        required: false,
        description: 'The account.',
    },
    {
        name: 'nonce',
        description: 'The nonce.',
        default: '0',
    },
];
AddressCommand.examples = ['eth ct:address 0x92970dbD5C0Ee6b439422bFd7cD71e1DDA921A03'];
AddressCommand.aliases = ['ct:a', 'ct:address'];
