"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const network_1 = require("../../base/network");
class CallCommand extends network_1.NetworkCommand {
    async run() {
        const { args, flags } = this.parse(CallCommand);
        let networkUrl;
        try {
            networkUrl = this.getNetworkUrl(flags);
            const { abi, methodCall, address } = args;
            const { contractCall } = await Promise.resolve().then(() => __importStar(require('../../helpers/contractCall')));
            const result = await contractCall(abi, methodCall, address, networkUrl);
            this.log(result);
        }
        catch (e) {
            this.error(e.message, { exit: 1 });
        }
    }
}
exports.default = CallCommand;
CallCommand.description = `Calls method <methodCall> in contract with abi <abi> at address <address>`;
CallCommand.flags = Object.assign({}, network_1.NetworkCommand.flags);
CallCommand.args = [
    {
        name: 'abi',
        required: true,
        description: `The contract's ABI.`,
    },
    {
        name: 'methodCall',
        required: true,
        description: `e.g.: 'myMethod(arg1,arg2,["a","b",3,["d","0x123..."]])'`,
    },
    {
        name: 'address',
        required: true,
        description: `The contract's  address.`,
    },
];
CallCommand.examples = [
    `eth contract:call --rinkeby erc20 'totalSupply()' 0x5592ec0cfb4dbc12d3ab100b257153436a1f0fea`,
];
