"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const network_1 = require("../../base/network");
const flags_1 = require("../../flags");
const checkCommandInputs_1 = require("../../helpers/checkCommandInputs");
const transactions_1 = require("../../helpers/transactions");
class DeployCommand extends network_1.NetworkCommand {
    async run() {
        const { args, flags } = this.parse(DeployCommand);
        if (checkCommandInputs_1.isEmptyCommand({}, args)) {
            this._help();
            this.exit(1);
        }
        let networkUrl;
        try {
            networkUrl = this.getNetworkUrl(flags);
            const { bin } = args;
            const { pk, 'confirmation-blocks': confirmationBlocks } = flags;
            if (!pk) {
                this.error('You have to specify a private key using --pk', { exit: 1 });
                return;
            }
            const { deploy } = await Promise.resolve().then(() => __importStar(require('../../helpers/deploy')));
            const txHash = await deploy(networkUrl, pk, bin);
            const receipt = await transactions_1.awaitTransactionMined(networkUrl, txHash, confirmationBlocks);
            cli_ux_1.cli.styledJSON({ address: receipt.contractAddress, receipt });
        }
        catch (e) {
            this.error(e.message, { exit: 1 });
        }
    }
}
exports.DeployCommand = DeployCommand;
DeployCommand.description = `Deploy contract whose bytecode is in <bin> using private key <pk>.`;
DeployCommand.flags = Object.assign(Object.assign({}, network_1.NetworkCommand.flags), { pk: flags_1.privateKeyFlag, 'confirmation-blocks': flags_1.confirmationBlocksFlag });
DeployCommand.args = [
    {
        name: 'bin',
        required: false,
        description: 'The bin file of the contract.',
    },
];
DeployCommand.examples = [
    'eth contract:deploy --ropsten --pk 3daa79a26454a5528a3523f9e6345efdbd636e63f8c24a835204e6ccb5c88f9e ./contracts/proxy.bin',
];
DeployCommand.aliases = ['ct:d', 'ct:deploy'];
