"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const ethereumjs_util_1 = require("ethereumjs-util");
const checkCommandInputs_1 = require("../../helpers/checkCommandInputs");
class HashCommand extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(HashCommand);
        if (checkCommandInputs_1.isEmptyCommand(flags, args)) {
            this._help();
            this.exit(1);
        }
        try {
            const { signature } = args;
            const hash = ethereumjs_util_1.keccak(signature)
                .toString('hex')
                .slice(0, 8);
            this.log(hash);
        }
        catch (e) {
            this.error(e.message, { exit: 1 });
        }
    }
}
exports.default = HashCommand;
HashCommand.description = `Get the hash of the given method.`;
HashCommand.args = [
    {
        name: 'signature',
        required: false,
        description: 'The given signature.',
    },
];
HashCommand.examples = [`eth method:hash 'transfer(address,uint256)'`];
HashCommand.aliases = ['m:hs', 'm:h'];
