"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const network_1 = require("../../base/network");
const flags_1 = require("../../flags");
const checkCommandInputs_1 = require("../../helpers/checkCommandInputs");
const transactions_1 = require("../../helpers/transactions");
class SendCommand extends network_1.NetworkCommand {
    async run() {
        const { args, flags } = this.parse(SendCommand);
        if (checkCommandInputs_1.isEmptyCommand({}, args)) {
            this._help();
            this.exit(1);
        }
        let networkUrl;
        try {
            networkUrl = this.getNetworkUrl(flags);
            const { abi, methodCall, address } = args;
            const { 'confirmation-blocks': confirmationBlocks, pk } = flags;
            const { encode } = await Promise.resolve().then(() => __importStar(require('../../helpers/encode')));
            const { sendTransaction } = await Promise.resolve().then(() => __importStar(require('../../helpers/sendTransaction')));
            const abiByteCode = encode(abi, methodCall, networkUrl);
            const tx = await sendTransaction(abiByteCode, address, pk, networkUrl);
            await transactions_1.awaitTransactionMined(networkUrl, tx, confirmationBlocks);
            cli_ux_1.cli.styledJSON(tx);
        }
        catch (e) {
            this.error(e.message, { exit: 1 });
        }
    }
}
exports.default = SendCommand;
SendCommand.description = `Executes <methodCall> for the contract in <address> given <abi> using the given private key.`;
SendCommand.flags = Object.assign(Object.assign({}, network_1.NetworkCommand.flags), { pk: Object.assign(Object.assign({}, flags_1.privateKeyFlag), { required: false }), 'confirmation-blocks': flags_1.confirmationBlocksFlag });
SendCommand.args = [
    {
        name: 'abi',
        required: false,
        description: `The contract's ABI.`,
    },
    {
        name: 'methodCall',
        required: false,
        description: `e.g.: 'myMethod(arg1,arg2,["a","b",3,["d","0x123..."]])'`,
    },
    {
        name: 'address',
        required: false,
        description: `The contract's  address.`,
    },
];
SendCommand.examples = [
    `eth method:send erc20 'transfer("0xb2E4a264A982039f8E503ea3C83af5537f583069", "0xDa480B4852ca3aDE4acF3eeCA6901952EdbAe912")' 0x15503FBAb2fa57535092ab9c24740142Ab6cabd3 0x6db0bdfc7800dcf87b5a88b3363997360395d36ef51db10c3458d51d8aefd37e`,
];
SendCommand.aliases = ['m:s'];
